////////////////////////////////////////
//  Texture methods
////////////////////////////////////////

GLuint	filter = GL_NEAREST;
GLuint	textureCube, textureSphere, textureCylinder, textureCone;

typedef struct {
    unsigned long sizeX;
    unsigned long sizeY;
    char *data;
}Image;


int loadImage(char *filename, Image *image) {

    // make sure the file is there.
    FILE *file;		
    if ((file = fopen(filename, "rb"))==NULL) {
				cout << "File Not Found : " << filename << endl;
				return 0;
    }
    
    // seek through the bmp header, up to the width/height:
    fseek(file, 18, SEEK_CUR);

    // read width and height, and calculate the size of the image
    unsigned long i;
    if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
				cout << "Error reading width from " << filename << endl;
				return 0;
    }    
    if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
				cout << "Error reading height from " << filename << endl;
				return 0;
    }    
		unsigned long size = image->sizeX * image->sizeY * 3;

    // read the planes
    unsigned short int planes;
    if ((fread(&planes, 2, 1, file)) != 1) {
				cout << "Error reading planes from " << filename << endl;
				return 0;
    }
    if (planes != 1) {
				cout << "Planes from " << filename << "is not 1: " << planes << endl;
				return 0;
    }

    // read the bpp
    unsigned short int bpp;
    if ((i = fread(&bpp, 2, 1, file)) != 1) { 
				cout << "Error reading bpp from " << filename << endl;
				return 0;
    }
    if (bpp != 24) {
				cout << "Bpp from " << filename << " is not 24: " << bpp << endl;
				return 0;
    }
	
    // seek past the rest of the bitmap header.
    fseek(file, 24, SEEK_CUR);

    // read the data. 
    image->data = (char *) malloc(size);
    if (image->data == NULL) {
				cout << "Error allocating memory for color-corrected image data" << endl;
				return 0;	
    }

    if ((i = fread(image->data, size, 1, file)) != 1) {
				cout << "Error reading image data from " << filename << endl;
				return 0;
    }

		// reverse all of the colors. (bgr -> rgb) 
		char temp;                         
		for (i=0;i<size;i+=3) { 
				temp = image->data[i];
				image->data[i] = image->data[i+2];
				image->data[i+2] = temp;
    }
		
    return 1;
}


GLvoid loadGLTexture(char* textureName, GLuint* textureIdx) {	
    // Load Texture
    Image *image1;
    
    // allocate space for cube texture
    image1 = (Image *) malloc(sizeof(Image));
    if (image1 == NULL) {
				printf("Error allocating space for image");
				exit(0);
    }
    if (!loadImage(textureName, image1)) {
				exit(1);
    }        

    // Create Texture
    glGenTextures(1, textureIdx);
    glBindTexture(GL_TEXTURE_2D, *textureIdx);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT); 
    glTexImage2D(GL_TEXTURE_2D, 0, 3, image1->sizeX, image1->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image1->data);
  
};


void drawSphere(){
		glBindTexture(GL_TEXTURE_2D, textureSphere);
		
		int nbStrips = 20;
		float stepVAngle = M_PI/nbStrips;
		
		int nbFacesPerStrip = 50;
		float stepHAngle = 2*M_PI/nbFacesPerStrip;

		for(int i=0 ; i<nbStrips ; i++) {
				float vAngle1 = i*stepVAngle - 0.5*M_PI;
				float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;
				
				// draw one strip
				float intensity1 = (float)i/nbStrips;
				float intensity2 = (float)(i+1)/nbStrips;
				glBegin(GL_QUAD_STRIP);
				for (int j=0 ; j<=nbFacesPerStrip ; j++){
						float hAngle = j*stepHAngle;						
						float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
						float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};
						
						glNormal3fv(pos1);
						glColor3f(intensity1, 0.0, 1.0-intensity1);
						glTexCoord2f(hAngle/(2.0*M_PI), i/(float)nbStrips);
						glVertex3fv(pos1);
						
						glNormal3fv(pos2);
						glColor3f(intensity2, 0.0, 1.0-intensity2);
						glTexCoord2f(hAngle/(2.0*M_PI), (i+1.0)/(float)nbStrips);
						glVertex3fv(pos2);
				}
				glEnd();
		}

}
